<?php

namespace Bixo\Chat\Http\Controllers;

use Exception;
use Bixo\Chat\Actions\ChatWorkflow;
use Bixo\Chat\Http\Requests\ChatWorkflowRequest;
use Bixo\Chat\Http\Resources\ChatResource;
use Bixo\Chat\Models\Chat;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class ChatWorkflowController extends BaseController
{
    /**
     * Action controller function for chat.
     *
     * @param Model $chat
     * @param action next action for the chat.
     *
     * @return Response
     */
    public function __invoke(ChatWorkflowRequest $request, Chat $chat, $trasition)
    {
        try {
            $request = $request->all();
            $chat = ChatWorkflow::run($trasition, $chat, $request);
            $data = new ChatResource($chat);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('chat::chat.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('chat/chat/' . $chat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('chat/chat/' . $chat->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the chat and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $chat
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $chat, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $chat = Chat::findBySignedId($chat);
        $transitions = $this->transitions($chat, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('chat::chat.name') . ' ' . trans('Approvals'))
            ->view('chat::public.chat.workflow')
            ->layout('mobile')
            ->data(compact('chat', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the chat.
     *
     * @param PublicRequest $request
     * @param string $chat
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $chat, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $chat = Chat::findBySignedId($chat);
        $user = user();
        $chat = ChatWorkflow::run($transition, $chat, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($chat, $user)
    {
        $transitions = [];
        foreach ($chat->workflow()->transitions($chat) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.chat.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'chat' => $chat->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $chat->workflow()->form($value);
            $array['label'] = trans('chat::chat.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
